/*
 * Copyright 2010 SpringSource
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

@IMPORT url("highlight.css");

html {
  padding:       0pt;
  margin:        0pt;
}

body {
  margin-left:   10%;
  margin-right:  10%;
  font-family:   Arial, Sans-serif;
}

div {
  margin:        0pt;
}

p {
  text-align:    justify;
}

hr {
  border:        1px solid gray;
  background:    gray;
}

h1,h2,h3,h4 {
  color:         #234623;
  font-family:   Arial, Sans-serif;
}

pre {
  line-height:   1.0;
  color:         black;
}

pre.programlisting {
  font-size:     10pt;
  padding:       7pt 3pt;
  border:        1pt solid black;
  background:    #eeeeee;
  clear:         both;
}

div.table {
  margin:        1em;
  padding:       0.5em;
  text-align:    center;
}

div.table table {
  display:       table;
  width:         100%;
}

div.table td {
  padding-left:  7px;
  padding-right: 7px;
}
