/*
 * Copyright 2010 SpringSource
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.hl-keyword {
  color: #7F0055;
  font-weight: bold; 
}

.hl-comment {
  color: #3F5F5F;
  font-style: italic;
}

.hl-multiline-comment {
  color: #3F5FBF;
  font-style: italic;
}

.hl-tag {
  color: #3F7F7F;
}

.hl-attribute {
  color: #7F007F;
}

.hl-value {
  color: #2A00FF;
}

.hl-string {
  color: #2A00FF;
}